"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk = require("chalk");
const cli_utils_1 = require("@ionic/cli-utils");
const CHOICE_CREATE_NEW_APP = 'createNewApp';
const CHOICE_NEVERMIND = 'nevermind';
let LinkCommand = class LinkCommand extends cli_utils_1.Command {
    preRun(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { create } = options;
            if (inputs[0] && create) {
                throw this.exit(`Sorry--cannot use both ${chalk.green('app_id')} and ${chalk.green('--create')}. You must either link an existing app or create a new one.`);
            }
            let proAppId = options['pro-id'] || '';
            const config = yield this.env.config.load();
            if (proAppId && config.backend !== cli_utils_1.BACKEND_PRO) {
                yield this.runcmd(['config', 'set', '-g', 'backend', 'pro'], { showExecution: false, showLogs: false });
                this.env.log.nl();
                this.env.log.info(`${chalk.bold(chalk.blue.underline('You have opted in to Ionic Pro!') + ' The CLI is now set up to use Ionic Pro services.')}\n` +
                    `You can revert back to Ionic Cloud (legacy) services at any time:\n\n` +
                    `${chalk.green('ionic config set -g backend legacy')}\n`);
                inputs[0] = proAppId;
            }
        });
    }
    run(inputs, options) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            let [appId] = inputs;
            let { create, name } = options;
            const config = yield this.env.config.load();
            const project = yield this.env.project.load();
            if (project.app_id) {
                const confirm = yield this.env.prompt({
                    type: 'confirm',
                    name: 'confirm',
                    message: `App ID ${chalk.green(project.app_id)} already exists in project config. Would you like to link a different app?`
                });
                if (!confirm) {
                    this.env.log.info('Not linking.');
                    return;
                }
            }
            if (!(yield this.env.session.isLoggedIn())) {
                yield cli_utils_1.promptToLogin(this.env);
            }
            if (config.backend === cli_utils_1.BACKEND_PRO) {
                if (!config.git.setup) {
                    const CHOICE_AUTOMATIC = 'automatic';
                    const CHOICE_MANUAL = 'manual';
                    const CHOICE_SKIP = 'skip';
                    const CHOICE_IGNORE = 'ignore';
                    const sshSetup = yield this.env.prompt({
                        type: 'list',
                        name: 'sshSetup',
                        message: `Looks like you haven't configured your SSH settings yet. How would you like to connect to Ionic Pro?`,
                        choices: [
                            {
                                name: 'Automatically setup new a SSH key pair for Ionic Pro',
                                value: CHOICE_AUTOMATIC,
                            },
                            {
                                name: 'Use an existing SSH key pair',
                                value: CHOICE_MANUAL,
                            },
                            {
                                name: 'Skip for now',
                                value: CHOICE_SKIP,
                            },
                            {
                                name: 'Ignore this prompt forever',
                                value: CHOICE_IGNORE,
                            },
                        ],
                    });
                    if (sshSetup === CHOICE_AUTOMATIC) {
                        yield this.runcmd(['ssh', 'setup']);
                    }
                    else if (sshSetup === CHOICE_MANUAL) {
                        yield this.runcmd(['ssh', 'add']);
                    }
                    else if (sshSetup === CHOICE_SKIP) {
                        this.env.log.warn(`Your first ${chalk.green('git push ionic master')} may not work.\n` +
                            `You can automatically configure your SSH settings using ${chalk.green('ionic ssh setup')}.`);
                    }
                    if (sshSetup !== CHOICE_SKIP) {
                        config.git.setup = true;
                    }
                }
            }
            if (appId) {
                if (appId === project.app_id) {
                    return this.env.log.info(`Already linked with app ${chalk.bold(appId)}.`);
                }
                this.env.tasks.next(`Looking up app ${chalk.bold(appId)}`);
                const token = yield this.env.session.getAppUserToken(appId);
                const req = this.env.client.make('GET', `/apps/${appId}`)
                    .set('Authorization', `Bearer ${token}`)
                    .send();
                const res = yield this.env.client.do(req);
                if (!cli_utils_1.isAppResponse(res)) {
                    throw cli_utils_1.createFatalAPIFormat(req, res);
                }
                this.env.tasks.end();
            }
            else if (!create) {
                this.env.tasks.next(`Looking up your apps`);
                let apps = [];
                const token = yield this.env.session.getUserToken();
                const paginator = this.env.client.paginate(() => this.env.client.make('GET', '/apps').set('Authorization', `Bearer ${token}`), cli_utils_1.isAppsResponse);
                for (let r of paginator) {
                    const res = yield r;
                    apps = apps.concat(res.data);
                }
                this.env.tasks.end();
                const createAppChoice = {
                    name: 'Create a new app',
                    id: CHOICE_CREATE_NEW_APP,
                };
                const neverMindChoice = {
                    name: 'Nevermind',
                    id: CHOICE_NEVERMIND,
                };
                const linkedApp = yield this.env.prompt({
                    type: 'list',
                    name: 'linkedApp',
                    message: `Which app would you like to link`,
                    choices: [createAppChoice, ...apps, neverMindChoice].map((app) => ({
                        name: [CHOICE_CREATE_NEW_APP, CHOICE_NEVERMIND].includes(app.id) ? chalk.bold(app.name) : `${app.name} (${app.id})`,
                        value: app.id
                    }))
                });
                appId = linkedApp;
            }
            if (create || appId === CHOICE_CREATE_NEW_APP) {
                const token = yield this.env.session.getUserToken();
                if (config.backend === cli_utils_1.BACKEND_PRO) {
                    if (!name) {
                        name = yield this.env.prompt({
                            type: 'input',
                            name: 'name',
                            message: 'Please enter a name for your new app:',
                        });
                    }
                    const req = this.env.client.make('POST', '/apps')
                        .set('Authorization', `Bearer ${token}`)
                        .send({ name });
                    const res = yield this.env.client.do(req);
                    if (!cli_utils_1.isAppResponse(res)) {
                        throw cli_utils_1.createFatalAPIFormat(req, res);
                    }
                    appId = res.data.id;
                    yield this.runcmd(['config', 'set', 'app_id', appId]);
                    yield this.runcmd(['git', 'remote']);
                    this.env.log.ok(`Project linked with app ${chalk.bold(appId)}!`);
                }
                else {
                    const opn = yield Promise.resolve().then(function () { return require('opn'); });
                    opn(`${config.urls.dash}/?user_token=${token}`, { wait: false });
                    this.env.log.info(`Rerun ${chalk.green(`ionic link`)} to link to the new app.`);
                }
            }
            else if (appId === CHOICE_NEVERMIND) {
                this.env.log.info('Not linking app.');
            }
            else {
                yield this.runcmd(['config', 'set', 'app_id', appId]);
                if (config.backend === cli_utils_1.BACKEND_PRO) {
                    yield this.runcmd(['git', 'remote']);
                }
                this.env.log.ok(`Project linked with app ${chalk.bold(appId)}!`);
            }
        });
    }
};
LinkCommand = tslib_1.__decorate([
    cli_utils_1.CommandMetadata({
        name: 'link',
        type: 'project',
        backends: [cli_utils_1.BACKEND_LEGACY, cli_utils_1.BACKEND_PRO],
        description: 'Connect your local app to Ionic',
        longDescription: `
If you have an app on Ionic, you can link it to this local Ionic project with this command.

Excluding the ${chalk.green('app_id')} argument looks up your apps on Ionic and prompts you to select one.

This command simply sets the ${chalk.bold('app_id')} property in ${chalk.bold(cli_utils_1.PROJECT_FILE)} for other commands to read.
  `,
        exampleCommands: ['', 'a1b2c3d4'],
        inputs: [
            {
                name: 'app_id',
                description: `The ID of the app to link (e.g. ${chalk.green('a1b2c3d4')})`,
                required: false,
            },
        ],
        options: [
            {
                name: 'name',
                description: 'The app name to use during the linking of a new app',
            },
            {
                name: 'create',
                description: 'Create a new app on Ionic and link it with this local Ionic project',
                backends: [cli_utils_1.BACKEND_PRO],
                type: Boolean,
                visible: false,
            },
            {
                name: 'pro-id',
                description: 'Specify an app ID from the Ionic Dashboard to link',
                visible: false,
            },
        ],
    })
], LinkCommand);
exports.LinkCommand = LinkCommand;
